/** @module swisstennis/state */
import { call, put, takeEvery } from 'redux-saga/effects'
import { SZTM_API } from '../local-constants'

/**
 * state.js
 *
 * Collection of everything which has to do with state changes.
 **/

/** actionTypes define what actions are handeled by the reducer. */
export const actions = {
  fileListRequest: () => {
    return {
      type: 'SWISSTENNIS/FILE_LIST_REQUEST',
    }
  },
  fileListSuccess: data => {
    return {
      type: 'SWISSTENNIS/FILE_LIST_SUCCESS',
      data
    }
  },
  fileListFailure: error => {
    return {
      type: 'SWISSTENNIS/FILE_LIST_FAILURE',
      error
    }
  },
  loginRequest: (data) => {
    return {
      type: 'SWISSTENNIS/LOGIN_REQUEST',
      data
    }
  },
  loginSuccess: (data) => {
    return {
      type: 'SWISSTENNIS/LOGIN_SUCCESS',
      data
    }
  },
  loginFailure: (error) => {
    return {
      type: 'SWISSTENNIS/LOGIN_FAILURE',
      error
    }
  },
  downloadPlayerListRequest: (data) => {
    return {
      type: 'SWISSTENNIS/DOWNLOAD_PLAYERLIST_REQUEST',
      data
    }
  },
  downloadPlayerListSuccess: (data) => {
    return {
      type: 'SWISSTENNIS/DOWNLOAD_PLAYERLIST_SUCCESS',
      data
    }
  },
  downloadPlayerListFailure: error => {
    return {
      type: 'SWISSTENNIS/DOWNLOAD_PLAYERLIST_FAILURE',
      error
    }
  },
  parsePlayerListRequest: (data) => {
    return {
      type: 'SWISSTENNIS/PARSE_PLAYERLIST_REQUEST',
      data
    }
  },
  parsePlayerListSuccess: (data) => {
    return {
      type: 'SWISSTENNIS/PARSE_PLAYERLIST_SUCCESS',
      data
    }
  },
  parsePlayerListFailure: error => {
    return {
      type: 'SWISSTENNIS/PARSE_PLAYERLIST_FAILURE',
      error
    }
  },
  downloadCalendarRequest: (data) => {
    return {
      type: 'SWISSTENNIS/DOWNLOAD_CALENDAR_REQUEST',
      data
    }
  },
  downloadCalendarSuccess: (data) => {
    return {
      type: 'SWISSTENNIS/DOWNLOAD_CALENDAR_SUCCESS',
      data
    }
  },
  downloadCalendarFailure: error => {
    return {
      type: 'SWISSTENNIS/DOWNLOAD_CALENDAR_FAILURE',
      error
    }
  },
  parseCalendarRequest: data => {
    return {
      type: 'SWISSTENNIS/PARSE_CALENDAR_REQUEST',
      data
    }
  },
  parseCalendarSuccess: data => {
    return {
      type: 'SWISSTENNIS/PARSE_CALENDAR_SUCCESS',
      data
    }
  },
  parseCalendarFailure: error => {
    return {
      type: 'SWISSTENNIS/PARSE_CALENDAR_FAILURE',
      error
    }
  },
  getPlayerListRequest: () => {
    return {
      type: 'SWISSTENNIS/GET_PLAYERLIST_REQUEST'
    }
  },
  getPlayerListSuccess: (data) => {
    return {
      type: 'SWISSTENNIS/GET_PLAYERLIST_SUCCESS',
      data
    }
  },
  getPlayerListFailure: error => {
    return {
      type: 'SWISSTENNIS/GET_PLAYERLIST_FAILURE',
      error
    }
  },
  getCalendarRequest: (data) => {
    return {
      type: 'SWISSTENNIS/GET_CALENDAR_REQUEST',
      data
    }
  },
  getCalendarSuccess: (data) => {
    return {
      type: 'SWISSTENNIS/GET_CALENDAR_SUCCESS',
      data
    }
  },
  getCalendarFailure: error => {
    return {
      type: 'SWISSTENNIS/GET_CALENDAR_FAILURE',
      error
    }
  },
}
console.log('State actions', actions)

/** state definition */
export const state = {
  files: [],
  fileListStatus: 'uninitialized',
  calendars: [],
  calendarsStatus: 'uninitialized',
  playerLists: [],
  playerListsStatus: 'uninitialized',
  playerListDownloadStatus: 'uninitialized',
  playerListParseStatus: 'uninitialized',
  calendarDownloadStatus: 'uninitialized',
  calendarParseStatus: 'uninitialized',
  loginStatus: 'uninitialized',
}
console.log('State state', state)

/** reducer is called by the redux dispatcher and handles all component actions */
export function reducer (state = [], action) {
  switch (action.type) {
    case 'SWISSTENNIS/FILE_LIST_REQUEST':
      return { ...state, fileListStatus: 'request' }
    case 'SWISSTENNIS/FILE_LIST_SUCCESS':
      return { ...state, fileListStatus: 'success', files: action.data }
    case 'SWISSTENNIS/FILE_LIST_FAILURE':
      return { ...state, fileListStatus: 'failure' }
    case 'SWISSTENNIS/LOGIN_REQUEST':
      return { ...state, 
        loginStatus: 'request', 
        playerListDownloadStatus: action.data.sequence ? 'uninitialized' : state.playerListDownloadStatus,
        playerListParseStatus: action.data.sequence ? 'uninitialized' : state.playerListParseStatus,
        calendarDownloadStatus: action.data.sequence ? 'uninitialized' : state.calendarDownloadStatus,
        calendarParseStatus: action.data.sequence ? 'uninitialized' : state.calendarParseStatus,
         }
    case 'SWISSTENNIS/LOGIN_SUCCESS':
      return { ...state, loginStatus: 'success' }
    case 'SWISSTENNIS/LOGIN_FAILURE':
      return { ...state, loginStatus: 'failure' }
    case 'SWISSTENNIS/DOWNLOAD_PLAYERLIST_REQUEST':
      return { ...state, playerListDownloadStatus: 'request' }
    case 'SWISSTENNIS/DOWNLOAD_PLAYERLIST_SUCCESS':
      return { ...state, playerListDownloadStatus: 'success', files: [ ...state.files, action.data ] }
    case 'SWISSTENNIS/DOWNLOAD_PLAYERLIST_FAILURE':
      return { ...state, playerListDownloadStatus: 'failure' }
    case 'SWISSTENNIS/PARSE_PLAYERLIST_REQUEST':
      return { ...state, playerListParseStatus: 'request' }
    case 'SWISSTENNIS/PARSE_PLAYERLIST_SUCCESS':
      return { ...state, playerListParseStatus: 'success', playerLists: [ ...state.playerLists, action.data.playerList ] }
    case 'SWISSTENNIS/PARSE_PLAYERLIST_FAILURE':
      return { ...state, playerListParseStatus: 'failure' }
    case 'SWISSTENNIS/DOWNLOAD_CALENDAR_REQUEST':
      return { ...state, calendarDownloadStatus: 'request' }
    case 'SWISSTENNIS/DOWNLOAD_CALENDAR_SUCCESS':
      return { ...state, calendarDownloadStatus: 'success', files: [ ...state.files, action.data ] }
    case 'SWISSTENNIS/DOWNLOAD_CALENDAR_FAILURE':
      return { ...state, calendarDownloadStatus: 'failure' }
    case 'SWISSTENNIS/PARSE_CALENDAR_REQUEST':
      return { ...state, calendarParseStatus: 'request' }
    case 'SWISSTENNIS/PARSE_CALENDAR_SUCCESS':
      return { ...state, calendarParseStatus: 'success', calendars: [ ...state.calendars, action.data.matchList ] }
    case 'SWISSTENNIS/PARSE_CALENDAR_FAILURE':
      return { ...state, calendarParseStatus: 'failure' }
    case 'SWISSTENNIS/GET_CALENDAR_REQUEST':
      return { ...state, calendarsStatus: 'loading' }
    case 'SWISSTENNIS/GET_CALENDAR_SUCCESS':
      return { ...state, calendarsStatus: 'loaded', calendars: action.data }
    case 'SWISSTENNIS/GET_CALENDAR_FAILURE':
      return { ...state, calendarsStatus: 'failed' }
      case 'SWISSTENNIS/GET_PLAYERLIST_REQUEST':
        return { ...state, playerListsStatus: 'loading' }
      case 'SWISSTENNIS/GET_PLAYERLIST_SUCCESS':
        return { ...state, playerListsStatus: 'loaded', playerLists: action.data }
      case 'SWISSTENNIS/GET_PLAYERLIST_FAILURE':
        return { ...state, playerListsStatus: 'failed' }
    default:
      return state
  }
}

function * fileList (action) {
  try {
    const token = localStorage.getItem('accessToken')
    console.log('File list requested', action, token)
    const response = yield call(fetch, `${SZTM_API}/api/swisstennis/files`, {
      method: 'GET',
      headers: {
        'Content-Type': 'application/json',
        'x-access-token': token
      }
    })
    if (response.status != 200) {
      console.log(response)
      throw Error(response.status)
    } else {
      const responseJson = yield response.json()
      yield put(actions.fileListSuccess(responseJson.fileList))
    }
  } catch (error) {
    console.log('Config failure!', actions.fileListFailure(error))
    yield put(actions.fileListFailure(error))
  }
}

function * login (action) {
  try {
    const token = localStorage.getItem('accessToken')
    console.log('Swisstennis login requested', action, token)
    const response = yield call(fetch, `${SZTM_API}/api/swisstennis/login`, {
      method: 'POST',
      headers: {
        'Content-Type': 'application/json',
        'x-access-token': token
      }
    }) 
    if (response.status != 200) {
      console.log(response)
      throw Error(response.status)
    } else {
      const responseJson = yield response.json()
      yield put(actions.loginSuccess(responseJson))
      if (action.data.sequence) yield put(actions.downloadPlayerListRequest({sequence: true}))
    }
  } catch (error) {
    console.log('Config failure!', actions.loginFailure(error))
    yield put(actions.loginFailure(error))
  }
}

function * getCalendars (action) {
  try {
    const token = localStorage.getItem('accessToken')
    console.log('Get config requested', action, token)
    const response = yield call(fetch, `${SZTM_API}/api/swisstennis/calendar`, {
      method: 'GET',
      headers: {
        'Content-Type': 'application/json',
        'x-access-token': token
      }
    }) 
    if (response.status != 200) {
      throw Error(response.status)
    } else {
      const responseJson = yield response.json()
      yield put(actions.getCalendarSuccess(responseJson.calendars))
    }
  } catch (error) {
    console.log('Config failure!', actions.getCalendarFailure(error))
    yield put(actions.getCalendarFailure(error))
  }
}

function * downloadCalendar (action) {
  try {
    const token = localStorage.getItem('accessToken')
    console.log('Download calendar requested', action, token)
    const response = yield call(fetch, `${SZTM_API}/api/swisstennis/calendar/download`, {
      method: 'GET',
      headers: {
        'Content-Type': 'application/json',
        'x-access-token': token
      }
    }) 
    if (response.status != 200) {
      throw Error(response.status)
    } else {
      const responseJson = yield response.json()
      yield put(actions.downloadCalendarSuccess(responseJson))
      if (action.data.sequence) yield put(actions.parseCalendarRequest({filename: responseJson.filename, sequence: true}))
    }
  } catch (error) {
    console.log('Config failure!', actions.downloadCalendarFailure(error))
    yield put(actions.downloadCalendarFailure(error))
  }
}

function * parseCalendar (action) {
  try {
    const token = localStorage.getItem('accessToken')
    console.log('Download calendar requested', action, token)
    const response = yield call(fetch, `${SZTM_API}/api/swisstennis/calendar/parse/${action.data.filename}`, {
      method: 'GET',
      headers: {
        'Content-Type': 'application/json',
        'x-access-token': token
      }
    }) 
    if (response.status != 200) {
      throw Error(response.status)
    } else {
      const responseJson = yield response.json()
      yield put(actions.parseCalendarSuccess(responseJson))
    }
  } catch (error) {
    console.log('Config failure!', actions.parseCalendarFailure(error))
    yield put(actions.parseCalendarFailure(error))
  }
}

function * getPlayerLists (action) {
  try {
    const token = localStorage.getItem('accessToken')
    console.log('Get config requested', action, token)
    const response = yield call(fetch, `${SZTM_API}/api/swisstennis/playerlist`, {
      method: 'GET',
      headers: {
        'Content-Type': 'application/json',
        'x-access-token': token
      }
    }) 
    if (response.status != 200) {
      throw Error(response.status)
    } else {
      const responseJson = yield response.json()
      yield put(actions.getPlayerListSuccess(responseJson.playerLists))
    }
  } catch (error) {
    console.log('Config failure!', actions.getPlayerListFailure(error))
    yield put(actions.getPlayerListFailure(error))
  }
}

function * downloadPlayerList (action) {
  try {
    const token = localStorage.getItem('accessToken')
    console.log('Playerlist download requested', action, token)
    const response = yield call(fetch, `${SZTM_API}/api/swisstennis/playerlist/download`, {
      method: 'GET',
      headers: {
        'x-access-token': token
      }
    }) 
    if (response.status != 200) {
      throw Error(response.status)
    } else {
      const responseJson = yield response.json()
      yield put(actions.downloadPlayerListSuccess(responseJson))
      if (action.data.sequence) yield put(actions.parsePlayerListRequest({filename: responseJson.filename, sequence: true}))
    }
  } catch (error) {
    console.log('Config failure!', actions.downloadPlayerListFailure(error))
    yield put(actions.downloadPlayerListFailure(error))
  }
}

function * parsePlayerList (action) {
  try {
    const token = localStorage.getItem('accessToken')
    console.log('PlayerList parse requested', action, token)
    const response = yield call(fetch, `${SZTM_API}/api/swisstennis/playerlist/parse/${action.data.filename}`, {
      method: 'GET',
      headers: {
        'x-access-token': token
      }
    })
    if (response.status != 200) {
      throw Error(response.status)
    } else {
      const responseJson = yield response.json()
      yield put(actions.parsePlayerListSuccess(responseJson))
      if (action.data.sequence) yield put(actions.downloadCalendarRequest({sequence: true}))
    }
  } catch (error) {
    console.log('Config failure!', actions.parsePlayerListFailure(error))
    yield put(actions.parsePlayerListFailure(error))
  }
}

/** sagas are asynchronous workers (JS generators) to handle the state. */
export function * saga () {
  console.log('Config saga started.')
  yield takeEvery('SWISSTENNIS/FILE_LIST_REQUEST', fileList)
  yield takeEvery('SWISSTENNIS/DOWNLOAD_PLAYERLIST_REQUEST', downloadPlayerList)
  yield takeEvery('SWISSTENNIS/PARSE_PLAYERLIST_REQUEST', parsePlayerList)
  yield takeEvery('SWISSTENNIS/DOWNLOAD_CALENDAR_REQUEST', downloadCalendar)
  yield takeEvery('SWISSTENNIS/PARSE_CALENDAR_REQUEST', parseCalendar)
  yield takeEvery('SWISSTENNIS/GET_CALENDAR_REQUEST', getCalendars)
  yield takeEvery('SWISSTENNIS/GET_PLAYERLIST_REQUEST', getPlayerLists)
  yield takeEvery('SWISSTENNIS/LOGIN_REQUEST', login)
}
